==> QEasyWidgets-0.6.7-py3-none-any.whl: QEasyWidgets/Common/QFunctions.py <==
import os
from pathlib import Path
from typing import Union, Optional
from PyEasyUtils import toIterable, normPath, runCMD
from PySide6.QtCore import Qt, QSettings, QPoint, QRect, QSize, QPropertyAnimation, QParallelAnimationGroup, QEasingCurve, QUrl
from PySide6.QtGui import QGuiApplication, QColor, QRgba64, QFont, QDesktopServices, QAction
from PySide6.QtWidgets import *

from ..Resources.Sources import *

##############################################################################################################################

def findChild(
    parent: QWidget,
    childType: object
):
    """
    Function to find child widget
    """
    parentWidget_Children = parent.children()

    for parentWidget_Child in parentWidget_Children:
        if isinstance(parentWidget_Child, childType):
            return parentWidget_Child


def findParent(
    child: QWidget,
    parentType: object
):
    """
    Function to find parent widget
    """
    childWidget_Parent = child.parent()

    while not isinstance(childWidget_Parent, parentType):
        try:
            childWidget_Parent = childWidget_Parent.parent()
        except:
            childWidget_Parent = None
            break

    return childWidget_Parent

##############################################################################################################################

def setFont(
    widget: QWidget,
    fontSize:int = 12,
    weight = QFont.Normal
):
    """
    Set the font of widget
    """
    Font = QFont()
    Font.setFamilies(['Microsoft YaHei'])
    Font.setPixelSize(fontSize)
    Font.setWeight(weight)
    widget.setFont(Font)

##############################################################################################################################

def setNoContents(
    widget: QWidget
):
    if isinstance(widget, QStackedWidget):
        while widget.count():
            widget.removeWidget(widget.widget(0))


def setRetainSizeWhenHidden(
    widget: QWidget,
    retainSize: bool = True
):
    sizePolicy = widget.sizePolicy()
    sizePolicy.setRetainSizeWhenHidden(retainSize)
    widget.setSizePolicy(sizePolicy)


def setDropShadowEffect(
    widget: QWidget,
    Radius: float = 3.,
    Color: Union[QColor, QRgba64] = Qt.gray
):
    DropShadowEffect = QGraphicsDropShadowEffect()
    DropShadowEffect.setOffset(0, 0)
    DropShadowEffect.setBlurRadius(Radius)
    DropShadowEffect.setColor(Color)
    widget.setGraphicsEffect(DropShadowEffect)

##############################################################################################################################

def setAnimation(
    animation: QPropertyAnimation,
    startValue,
    endValue,
    duration: int
):
    animation.setStartValue(startValue)
    animation.setEndValue(endValue)
    animation.setDuration(duration)
    animation.setEasingCurve(QEasingCurve.InOutQuart)
    return animation


def setWidgetPosAnimation(
    widget: QWidget,
    duration: int = 99
):
    OriginalGeometry = widget.geometry()
    AlteredGeometry = QRect(OriginalGeometry.left(), OriginalGeometry.top() + OriginalGeometry.height() / duration, OriginalGeometry.width(), OriginalGeometry.height())

    WidgetAnimation = QPropertyAnimation(widget, b"geometry", widget)

    return setAnimation(WidgetAnimation, OriginalGeometry, AlteredGeometry, duration)


def setWidgetSizeAnimation(
    frame: QWidget,
    targetWidth: Optional[int] = None,
    targetHeight: Optional[int] = None,
    duration: int = 210,
    supportSplitter: bool = False
):
    """
    Function to animate widget size
    """
    CurrentWidth = frame.geometry().width() if frame.size() == QSize(100, 30) else frame.width()
    CurrentHeight = frame.geometry().height() if frame.size() == QSize(100, 30) else frame.height()

    FrameAnimationMinWidth = QPropertyAnimation(frame, b"minimumWidth", frame)
    FrameAnimationMaxWidth = QPropertyAnimation(frame, b"maximumWidth", frame)
    FrameAnimationMinHeight = QPropertyAnimation(frame, b"minimumHeight", frame)
    FrameAnimationMaxHeight = QPropertyAnimation(frame, b"maximumHeight", frame)

    AnimationGroup = QParallelAnimationGroup(frame)

    AnimationGroup.addAnimation(setAnimation(FrameAnimationMinWidth, CurrentWidth, targetWidth, duration)) if targetWidth is not None and not supportSplitter else None
    AnimationGroup.addAnimation(setAnimation(FrameAnimationMaxWidth, CurrentWidth, targetWidth, duration)) if targetWidth is not None else None
    AnimationGroup.addAnimation(setAnimation(FrameAnimationMinHeight, CurrentHeight, targetHeight, duration)) if targetHeight is not None and not supportSplitter else None
    AnimationGroup.addAnimation(setAnimation(FrameAnimationMaxHeight, CurrentHeight, targetHeight, duration)) if targetHeight is not None else None

    return AnimationGroup


def setWidgetOpacityAnimation(
    widget: QWidget,
    originalOpacity: float,
    targetOpacity: float,
    duration: int = 99
):
    OpacityEffect = QGraphicsOpacityEffect()
    widget.setGraphicsEffect(OpacityEffect)

    originalOpacity = originalOpacity
    AlteredOpacity = targetOpacity

    WidgetAnimation = QPropertyAnimation(OpacityEffect, b"opacity", widget)

    return setAnimation(WidgetAnimation, originalOpacity, AlteredOpacity, duration)

##############################################################################################################################

def setText(
    widget: QWidget,
    text: str,
    setHtml: bool = True,
    setPlaceholderText: bool = False,
    placeholderText: Optional[str] = None
):
    if hasattr(widget, 'setText'):
        widget.setText(text)
    if hasattr(widget, 'setPlainText'):
        widget.setPlainText(text)
    if hasattr(widget, 'setHtml') and setHtml:
        widget.setHtml(text)
    if hasattr(widget, 'setPlaceholderText') and setPlaceholderText:
        widget.setPlaceholderText(str(placeholderText) if text.strip() in ('', str(None)) else text)


def getText(
    widget: QWidget,
    getHtml: bool = False,
    getPlaceholderText: bool = False
):
    if hasattr(widget, 'text'):
        text = widget.text()
    if hasattr(widget, 'toPlainText'):
        text = widget.toPlainText()
    if hasattr(widget, 'toHtml') and getHtml:
        text = widget.toHtml()
    if hasattr(widget, 'placeholderText') and getPlaceholderText:
        text = widget.placeholderText() if text.strip() in ('', str(None)) else text
    return text

##############################################################################################################################

class FileDialogMode: 
    SelectFolder = 0
    SelectFile = 1
    SaveFile = 2


def getFileDialog(
    mode: FileDialogMode,
    fileType: Optional[str] = None,
    directory: Optional[str] = None
):
    displayText = ""
    os.makedirs(directory, exist_ok = True) if directory is not None and Path(directory).exists() == False else None
    if mode == FileDialogMode.SelectFolder:
        displayText = QFileDialog.getExistingDirectory(
            caption = "选择文件夹 | SelectFolder",
            dir = directory if directory is not None else os.getcwd()
        )
    if mode == FileDialogMode.SelectFile:
        displayText, _ = QFileDialog.getOpenFileName(
            caption = "选择文件 | SelectFile",
            dir = directory if directory is not None else os.getcwd(),
            filter = fileType if fileType is not None else '任意类型 (*.*)'
        )
    if mode == FileDialogMode.SaveFile:
        displayText, _ = QFileDialog.getSaveFileName(
            caption = "保存文件 | SaveFile",
            dir = directory if directory is not None else os.getcwd(),
            filter = fileType if fileType is not None else '任意类型 (*.*)'
        )
    return displayText# if displayText != '' else None

##############################################################################################################################

def setContextMenu(parent: QWidget, contextMenu: QMenu, actions: dict):
    for actionName, events in actions.items():
        action = QAction(actionName, parent)
        for event in toIterable(events):
            action.triggered.connect(event)
        contextMenu.addAction(action)
    return contextMenu


def showContextMenu(parent: QWidget, contextMenu: QMenu, actions: dict, position: QPoint):
    contextMenu = setContextMenu(parent, contextMenu, actions)
    contextMenu.exec(position)

##############################################################################################################################

def openURL(
    url: Union[str, list],
    createIfNotExist: bool = False
):
    """
    Function to open web/local url
    """
    def OpenURL(url):
        QURL = QUrl().fromLocalFile(normPath(url))
        if QURL.isValid():
            os.makedirs(normPath(url), exist_ok = True) if createIfNotExist else None
            isSucceeded = QDesktopServices.openUrl(QURL)
            runCMD([f'start {url}']) if not isSucceeded else None
        else:
            print(f"Invalid url: {url} !")

    if isinstance(url, str):
        OpenURL(url)
    else:
        URLList = toIterable(url)
        for Index, url in enumerate(URLList):
            #url = Function_ParamsChecker(URLList)[Index] if isinstance(url, QObject) else url
            OpenURL(url)

##############################################################################################################################

def saveLayout(widget: QWidget, settings: QSettings):
    settings.setValue("layout/geometry", widget.saveGeometry())
    if isinstance(widget, QMainWindow):
        settings.setValue("layout/state", widget.saveState())
    settings.sync()


def resetLayout(widget: QWidget, settings: QSettings):
    widget.restoreGeometry(settings.value("layout/geometry"))
    if isinstance(widget, QMainWindow):
        widget.restoreState(settings.value("layout/state"))
        for dockWidget in widget.findChildren(QDockWidget):
            widget.restoreDockWidget(dockWidget)

##############################################################################################################################

def getCurrentScreen():
    """ get current screen """
    cursorPos = QCursor.pos()

    for s in QApplication.screens():
        if s.geometry().contains(cursorPos):
            return s

    return None


def getCurrentScreenGeometry(avaliable=True):
    """ get current screen geometry """
    screen = getCurrentScreen() or QApplication.primaryScreen()

    # this should not happen
    if not screen:
        return QRect(0, 0, 1920, 1080)

    return screen.availableGeometry() if avaliable else screen.geometry()

==> QEasyWidgets-0.7.0-py3-none-any.whl: QEasyWidgets/Common/QFunctions.py <==
import os
from pathlib import Path
from typing import Union, Optional
from PyEasyUtils import toIterable, normPath, runCMD
from PySide6.QtCore import Qt, QSettings, QPoint, QRect, QSize, QPropertyAnimation, QParallelAnimationGroup, QEasingCurve, QUrl
from PySide6.QtGui import QGuiApplication, QColor, QRgba64, QFont, QScreen, QDesktopServices, QAction, QCursor
from PySide6.QtWidgets import *

from ..Resources.Sources import *

##############################################################################################################################

def findChild(
    parent: QWidget,
    childType: object
):
    """
    Function to find child widget
    """
    parentWidget_Children = parent.children()

    for parentWidget_Child in parentWidget_Children:
        if isinstance(parentWidget_Child, childType):
            return parentWidget_Child


def findParent(
    child: QWidget,
    parentType: object
):
    """
    Function to find parent widget
    """
    childWidget_Parent = child.parent()

    while not isinstance(childWidget_Parent, parentType):
        try:
            childWidget_Parent = childWidget_Parent.parent()
        except:
            childWidget_Parent = None
            break

    return childWidget_Parent

##############################################################################################################################

def setFont(
    widget: QWidget,
    fontSize:int = 12,
    weight = QFont.Normal
):
    """
    Set the font of widget
    """
    Font = QFont()
    Font.setFamilies(['Microsoft YaHei'])
    Font.setPixelSize(fontSize)
    Font.setWeight(weight)
    widget.setFont(Font)

##############################################################################################################################

def setNoContents(
    widget: QWidget
):
    if isinstance(widget, QStackedWidget):
        while widget.count():
            widget.removeWidget(widget.widget(0))


def setRetainSizeWhenHidden(
    widget: QWidget,
    retainSize: bool = True
):
    sizePolicy = widget.sizePolicy()
    sizePolicy.setRetainSizeWhenHidden(retainSize)
    widget.setSizePolicy(sizePolicy)


def setDropShadowEffect(
    widget: QWidget,
    Radius: float = 3.,
    Color: Union[QColor, QRgba64] = Qt.gray
):
    DropShadowEffect = QGraphicsDropShadowEffect()
    DropShadowEffect.setOffset(0, 0)
    DropShadowEffect.setBlurRadius(Radius)
    DropShadowEffect.setColor(Color)
    widget.setGraphicsEffect(DropShadowEffect)

##############################################################################################################################

def setAnimation(
    animation: QPropertyAnimation,
    startValue,
    endValue,
    duration: int
):
    animation.setStartValue(startValue)
    animation.setEndValue(endValue)
    animation.setDuration(duration)
    animation.setEasingCurve(QEasingCurve.InOutQuart)
    return animation


def setWidgetPosAnimation(
    widget: QWidget,
    duration: int = 99
):
    OriginalGeometry = widget.geometry()
    AlteredGeometry = QRect(OriginalGeometry.left(), OriginalGeometry.top() + OriginalGeometry.height() / duration, OriginalGeometry.width(), OriginalGeometry.height())

    WidgetAnimation = QPropertyAnimation(widget, b"geometry", widget)

    return setAnimation(WidgetAnimation, OriginalGeometry, AlteredGeometry, duration)


def setWidgetSizeAnimation(
    frame: QWidget,
    targetWidth: Optional[int] = None,
    targetHeight: Optional[int] = None,
    duration: int = 210,
    supportSplitter: bool = False
):
    """
    Function to animate widget size
    """
    CurrentWidth = frame.geometry().width() if frame.size() == QSize(100, 30) else frame.width()
    CurrentHeight = frame.geometry().height() if frame.size() == QSize(100, 30) else frame.height()

    FrameAnimationMinWidth = QPropertyAnimation(frame, b"minimumWidth", frame)
    FrameAnimationMaxWidth = QPropertyAnimation(frame, b"maximumWidth", frame)
    FrameAnimationMinHeight = QPropertyAnimation(frame, b"minimumHeight", frame)
    FrameAnimationMaxHeight = QPropertyAnimation(frame, b"maximumHeight", frame)

    AnimationGroup = QParallelAnimationGroup(frame)

    AnimationGroup.addAnimation(setAnimation(FrameAnimationMinWidth, CurrentWidth, targetWidth, duration)) if targetWidth is not None and not supportSplitter else None
    AnimationGroup.addAnimation(setAnimation(FrameAnimationMaxWidth, CurrentWidth, targetWidth, duration)) if targetWidth is not None else None
    AnimationGroup.addAnimation(setAnimation(FrameAnimationMinHeight, CurrentHeight, targetHeight, duration)) if targetHeight is not None and not supportSplitter else None
    AnimationGroup.addAnimation(setAnimation(FrameAnimationMaxHeight, CurrentHeight, targetHeight, duration)) if targetHeight is not None else None

    return AnimationGroup


def setWidgetOpacityAnimation(
    widget: QWidget,
    originalOpacity: float,
    targetOpacity: float,
    duration: int = 99
):
    OpacityEffect = QGraphicsOpacityEffect()
    widget.setGraphicsEffect(OpacityEffect)

    originalOpacity = originalOpacity
    AlteredOpacity = targetOpacity

    WidgetAnimation = QPropertyAnimation(OpacityEffect, b"opacity", widget)

    return setAnimation(WidgetAnimation, originalOpacity, AlteredOpacity, duration)

##############################################################################################################################

def setText(
    widget: QWidget,
    text: str,
    setHtml: bool = True,
    setPlaceholderText: bool = False,
    placeholderText: Optional[str] = None
):
    if hasattr(widget, 'setText'):
        widget.setText(text)
    if hasattr(widget, 'setPlainText'):
        widget.setPlainText(text)
    if hasattr(widget, 'setHtml') and setHtml:
        widget.setHtml(text)
    if hasattr(widget, 'setPlaceholderText') and setPlaceholderText:
        widget.setPlaceholderText(str(placeholderText) if text.strip() in ('', str(None)) else text)


def getText(
    widget: QWidget,
    getHtml: bool = False,
    getPlaceholderText: bool = False
):
    if hasattr(widget, 'text'):
        text = widget.text()
    if hasattr(widget, 'toPlainText'):
        text = widget.toPlainText()
    if hasattr(widget, 'toHtml') and getHtml:
        text = widget.toHtml()
    if hasattr(widget, 'placeholderText') and getPlaceholderText:
        text = widget.placeholderText() if text.strip() in ('', str(None)) else text
    return text

##############################################################################################################################

class FileDialogMode: 
    SelectFolder = 0
    SelectFile = 1
    SaveFile = 2


def getFileDialog(
    mode: FileDialogMode,
    fileType: Optional[str] = None,
    directory: Optional[str] = None
):
    displayText = ""
    os.makedirs(directory, exist_ok = True) if directory is not None and Path(directory).exists() == False else None
    if mode == FileDialogMode.SelectFolder:
        displayText = QFileDialog.getExistingDirectory(
            caption = "选择文件夹 | SelectFolder",
            dir = directory if directory is not None else os.getcwd()
        )
    if mode == FileDialogMode.SelectFile:
        displayText, _ = QFileDialog.getOpenFileName(
            caption = "选择文件 | SelectFile",
            dir = directory if directory is not None else os.getcwd(),
            filter = fileType if fileType is not None else '任意类型 (*.*)'
        )
    if mode == FileDialogMode.SaveFile:
        displayText, _ = QFileDialog.getSaveFileName(
            caption = "保存文件 | SaveFile",
            dir = directory if directory is not None else os.getcwd(),
            filter = fileType if fileType is not None else '任意类型 (*.*)'
        )
    return displayText# if displayText != '' else None

##############################################################################################################################

def setContextMenu(parent: QWidget, contextMenu: QMenu, actions: dict):
    for actionName, events in actions.items():
        action = QAction(actionName, parent)
        for event in toIterable(events):
            action.triggered.connect(event)
        contextMenu.addAction(action)
    return contextMenu


def showContextMenu(parent: QWidget, contextMenu: QMenu, actions: dict, position: QPoint):
    contextMenu = setContextMenu(parent, contextMenu, actions)
    contextMenu.exec(position)

##############################################################################################################################

def openURL(
    url: Union[str, list],
    createIfNotExist: bool = False
):
    """
    Function to open web/local url
    """
    def OpenURL(url):
        QURL = QUrl().fromLocalFile(normPath(url))
        if QURL.isValid():
            os.makedirs(normPath(url), exist_ok = True) if createIfNotExist else None
            isSucceeded = QDesktopServices.openUrl(QURL)
            runCMD([f'start {url}']) if not isSucceeded else None
        else:
            print(f"Invalid url: {url} !")

    if isinstance(url, str):
        OpenURL(url)
    else:
        URLList = toIterable(url)
        for Index, url in enumerate(URLList):
            #url = Function_ParamsChecker(URLList)[Index] if isinstance(url, QObject) else url
            OpenURL(url)

##############################################################################################################################

def saveLayout(widget: QWidget, settings: QSettings):
    settings.setValue("layout/geometry", widget.saveGeometry())
    if isinstance(widget, QMainWindow):
        settings.setValue("layout/state", widget.saveState())
    settings.sync()


def resetLayout(widget: QWidget, settings: QSettings):
    widget.restoreGeometry(settings.value("layout/geometry"))
    if isinstance(widget, QMainWindow):
        widget.restoreState(settings.value("layout/state"))
        for dockWidget in widget.findChildren(QDockWidget):
            widget.restoreDockWidget(dockWidget)

##############################################################################################################################

def getCurrentScreen():
    """
    Get current screen
    """
    cursorPos = QCursor.pos()
    for screen in QApplication.screens():
        if screen.geometry().contains(cursorPos):
            return screen


def getScreenGeometry(
    screen: Optional[QScreen] = None,
    getAvaliableGeometry: bool = True
):
    """
    Get screen geometry
    """
    if screen is None:
        screen = getCurrentScreen() or QApplication.primaryScreen()
    return screen.availableGeometry() if getAvaliableGeometry else screen.geometry()

##############################################################################################################################
